#!/usr/bin/python3
import requests
import rospy
import numpy as np
from sensor_msgs.msg import NavSatFix
import json
import datetime
import time

class SqlHandler():

    def callback(self, data, args):
        self.deviceId = args[0]
        self.taskId = args[1]
        self.authorization = args[2]
        self.baseTable = args[3]
        self.gpsTable = args[4]
        if self.firstGpsFlag:
            self.firstGpsFlag = False
            self.gps_data = {
                'gps' : [
                    {
                    'stamp' : data.header.seq,
                    'altitude' : data.altitude,
                    'lat' : data.latitude,
                    'long' : data.longitude
                    },
                ]
            }

            self.gpsData = {'deviceid':self.deviceId, 'taskid':self.taskId, 'gpsdata' : json.dumps(self.gps_data)}

            self.r = requests.post(self.baseTable + self.gpsTable,auth=self.authorization, data = self.gpsData)
            print("gps post request response: " + self.r.text)

        if data.header.seq % 10 == 0: # only update every 10th message to avoid over crowding the sql
            self.gps_data = {
                'gps' : [
                    {
                    'stamp' : data.header.seq,
                    'altitude' : data.altitude,
                    'lat' : data.latitude,
                    'long' : data.longitude
                    },
                ]
            }

            self.gpsData = {'deviceid':self.deviceId, 'taskid':self.taskId, 'gpsdata' : json.dumps(self.gps_data)}
            self.r = requests.patch(self.baseTable + self.gpsTable + self.deviceId + '/',auth=self.authorization, data = self.gpsData)

    def __init__(self):
        self.firstGpsFlag = True
        # self.baseTable = 'http://192.168.1.230:8000'
        self.baseTable = 'http://127.0.0.1:8000'
        self.gpsTable = '/gpsdatas/'
        self.waypointTable = '/waypointsdata/'
        self.authorization = ('larkinheintzman','password')
        self.deviceId = str(rospy.get_param("drone_id"))
        self.taskId = 'task_0'
        self.topic = 'dji_sdk/gps_position'

        rospy.init_node('sql_uploader', anonymous = True)
        rospy.Subscriber(self.topic, NavSatFix, self.callback, (self.deviceId, self.taskId,
        self.authorization, self.baseTable, self.gpsTable), queue_size = 1)
        # self.gpsSpoofer = rospy.Publisher(self.topic, NavSatFix, queue_size = 1)
        self.updateRate = rospy.Rate(10)

        while not rospy.has_param('/robot_list/robot_' + str(self.deviceId) + '/waypoints'):
            print("waiting for parameter...")
            self.updateRate.sleep()

        # upload waypoints to GUI
        self.waypoint_dict = rospy.get_param('/robot_list/robot_' + str(self.deviceId) + '/waypoints')
        self.waypoint_json = []

        for i,pt in enumerate(self.waypoint_dict.keys()):
            self.waypoint_json.append({
            'altitude' : self.waypoint_dict[pt][3],
            'lat' : self.waypoint_dict[pt][1],
            'long' : self.waypoint_dict[pt][2],
            'direction' : self.waypoint_dict[pt][4],
            'timestamp' : datetime.datetime.now().strftime("%m/%d/%Y, %H:%M:%S")
            })
            # waypoint_json = sorted(waypoint_json, key = lambda i: i['stamp'])

            self.waypointData = {'deviceid':self.deviceId, 'taskid':self.taskId, 'waypointsdata' : json.dumps(self.waypoint_json)}
            # post new vehicle
            self.r = requests.post(self.baseTable + self.waypointTable, auth=self.authorization, data = self.waypointData)
            print("waypoint post request response: " + self.r.text)

            # patch old vehicle's waypoints
            self.r = requests.patch(self.baseTable + self.waypointTable + self.deviceId + '/', auth=self.authorization, data = self.waypointData)
            # print("waypoint patch request response: " + self.r.text)

        while not rospy.is_shutdown():
            self.updateRate.sleep()

if __name__ == '__main__':
    SqlHandler()
